import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.atomic.AtomicInteger;

public class MonteCarloMultiThreadDemo {

	private static AtomicInteger val = new AtomicInteger(0);
	private static int N = 100000000;
	
	public static void main(String[] args) {
		
		ExecutorService pool = Executors.newFixedThreadPool(1);
		
		long t0 = System.currentTimeMillis();
		
		for (int i = 1; i <= N; i++) {
			
			// single thread
			new Sample().run();
			
			// multithread ver1
//			pool.execute(new Sample());
			
			// multithread ver2
//			pool.execute(() -> {
//				
//				double x = ThreadLocalRandom.current().nextDouble();
//				double y = ThreadLocalRandom.current().nextDouble();
//				
//				if (x * x + y * y < 1) val.incrementAndGet();
//				
//			});
			
		}
		pool.shutdown();
		while (!pool.isTerminated());
		
		long t1 = System.currentTimeMillis();
		
		System.out.println(4.0 * val.get() / N);
		System.out.println(t1 - t0 + " ms");
		
	}
	
	private static class Sample implements Runnable {
		
		@Override
		public void run() {
			
			double x = ThreadLocalRandom.current().nextDouble();
			double y = ThreadLocalRandom.current().nextDouble();
			if (x * x + y * y < 1) val.incrementAndGet();
			
		}
		
	}

}
