import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.atomic.AtomicInteger;

public class MonteCarloMultiThreadDemo2 {
	
	private final static int nThrows = 100000000;
	
	public static void main(String[] args) {
		
		System.out.println("<<<<---- Single-Thread Simulation ---->>>>");
		scenario1();
		System.out.println("<<<<---- Multiple-Thread Simulation ---->>>>");
		scenario2();

	}
	
	public static void scenario1() {
		
		long startTime = System.currentTimeMillis();
		
		int nSuccess = 0;
		for (int i = 1; i <= nThrows; i++) {
			double x = ThreadLocalRandom.current().nextDouble();
			double y = ThreadLocalRandom.current().nextDouble();
			if (x * x + y * y <= 1) nSuccess++;
		}
		double value = 4.0 * nSuccess / nThrows;
		long stopTime = System.currentTimeMillis();
		
		System.out.println("Approx value: " + value);
		System.out.println("Difference to exact value of pi: " + (value - Math.PI));
		System.out.println("Error: " + (value - Math.PI) / Math.PI * 100 + " %");
		System.out.println("Time duration: " + (stopTime - startTime) + "ms");
		
	}
	
	public static void scenario2() {
		
		PiMonteCarlo simulator = new PiMonteCarlo(nThrows);
		long startTime = System.currentTimeMillis();
		double value = simulator.getPi();
		long stopTime = System.currentTimeMillis();
		
		System.out.println("Approx value: " + value);
		System.out.println("Difference to exact value of pi: " + (value - Math.PI));
		System.out.println("Error: " + (value - Math.PI) / Math.PI * 100 + " %");
		System.out.println("Time duration: " + (stopTime - startTime) + "ms");
		
	}
	
	private static class PiMonteCarlo {
		
		AtomicInteger nAtomSuccess;
		int nThrows;
		
		class Sample implements Runnable {
			
			@Override
			public void run() {
				
				double x = ThreadLocalRandom.current().nextDouble();
				double y = ThreadLocalRandom.current().nextDouble();
				if (x * x + y * y <= 1) nAtomSuccess.incrementAndGet();
				
			}
			
		}
		
		public PiMonteCarlo(int num) {
			
			this.nAtomSuccess = new AtomicInteger(0);
			this.nThrows = num;
			
		}
		
		public double getPi() {
			
			int nProcessors = Runtime.getRuntime().availableProcessors();
			ExecutorService executor = Executors.newWorkStealingPool(nProcessors);
//			for (int i = 1; i <= nThrows; i++) executor.execute(new Sample());
			
			for (int i = 1; i <= nThrows; i++) {
				
				Runnable task = () -> {
					double x = ThreadLocalRandom.current().nextDouble();
					double y = ThreadLocalRandom.current().nextDouble();
					if (x * x + y * y <= 1) nAtomSuccess.incrementAndGet();
				};
				
				executor.execute(task);
			}
			
			executor.shutdown();
			while (!executor.isTerminated());
			return 4.0 * nAtomSuccess.get() / nThrows;
			
		}
		
	}
}