import java.util.stream.Stream;
import java.util.Arrays;
import java.time.LocalTime;
 
public class MonteCarloMultiThreadDemo3 {
	
    private final static Object lock = new Object();
    
    private final static int TotalSamples = 10000000;
    private static int total = 0;
 
    private static Stream<Integer> prepare(int threadNumber) {
    	
        Integer[] Samples = new Integer[threadNumber];
        Arrays.fill(Samples, TotalSamples / threadNumber);
        return threadNumber > 1 ? Arrays.stream(Samples).parallel() : Arrays.stream(Samples).sequential();
        
    }
    
    public static void main (String[] args) {
    	
        System.out.println("-------\nRunning sequential\n-------");
        run(prepare(1));
 
        System.out.println("-------\nRunning parallel\n-------");
        run(prepare(8));
        
    }
 
    public static void run (Stream<Integer> stream) {
    	
        long startTime = System.nanoTime();
        
        total = 0;
        stream.forEach(s -> {
        	
            System.out.println(LocalTime.now() + " - value: " + s + " - thread: " + Thread.currentThread().getName());            
            int n = 0;
            for (int i = 0; i < s; ++i) {
                double x = Math.random();
                double y = Math.random();
                if (x * x + y * y <= 1) n++;                            
            }
            
            synchronized(lock) {
                total += n;
            }
            
        });
        
        long endTime = System.nanoTime();
        System.out.println(((endTime - startTime) / 1000000.0) + " ms");
        System.out.println("PI = " + total * 4.0 / TotalSamples);
        
    }
    
}