import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class MonteCarloPI {

	public static void main(String[] args) {
		
		int n = 10000000;
//		List<Point> list = new ArrayList<Point>();
//		for (int i = 1; i <= n; i++) list.add(Point.random());
		List<Point> points = Stream.iterate(0, i -> i + 1).limit(n).map(i -> Point.random()).collect(Collectors.toList());
		Predicate<Point> condition = p -> Math.pow(p.x, 2) + Math.pow(p.y, 2) < 1;
		
		long t0, t1;
		double pi, dt;
		
		System.out.println("---- simple for loop ----");
		t0 = System.nanoTime();
		pi = 0;
		for (Point p: points)if (condition.test(p)) pi++;
		pi = pi * 4 / n;
		t1 = System.nanoTime();
	    dt = (t1 - t0) / 1e6;
		System.out.println(pi + " in " + dt + " ms.");
		
		System.out.println("---- sequential stream ----");
		t0 = System.nanoTime();
		pi = points.stream().filter(condition).count() * 4.0 / n;
		t1 = System.nanoTime();
	    dt = (t1 - t0) / 1e6;
		System.out.println(pi + " in " + dt + " ms.");
		
		System.out.println("---- parallel stream ----");
		t0 = System.nanoTime();
		pi = points.parallelStream().filter(p -> Math.pow(p.x, 2) + Math.pow(p.y, 2) < 1).count() * 4.0 / n;
		t1 = System.nanoTime();
	    dt = (t1 - t0) / 1e6;
		System.out.println(pi + " in " + dt + " ms.");
		
	}
	
}

class Point {
	
	final double x, y;

	private Point(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	static Point random() {
		return new Point(Math.random(), Math.random());
	}
}