import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

public class MonteCarloPI2 {

	
	public static void main(String[] args) throws InterruptedException {
		
		Worker[] workers = new Worker[8];
		List<Point> points = gen(1000000); 
		
		long t0 = System.currentTimeMillis();
		deploy(workers, points, p -> Math.pow(p.x, 2) + Math.pow(p.y, 2) < 1);
		double pi = 0;
		for (Worker w: workers) pi += w.getResult();
		pi = pi * 4 / points.size();
		long duration = System.currentTimeMillis() - t0;
		
		System.out.println("time cost = " + duration + " ms");
		System.out.println("approx. pi = " + pi);
		
	}
	
	public static List<Point> gen(int N) {
		
		List<Point> points = new ArrayList<Point>();
		for (int i = 0; i < N; i++) points.add(Point.random());
		return points;
		
	}
	
	public static void deploy(Worker[] workers, List<Point> points, Predicate<Point> predicate) throws InterruptedException {
		
		int n = points.size() / workers.length; 
		int start = 0;
		Worker w = null;
		for (int i = 0; i < workers.length; i++) {
			w = new Worker(new Job(points, start, start + n - 1, predicate));
			w.start();
			w.join();
			workers[i] = w;
			start += n;
		}
		
	}
	
}

class Worker extends Thread {
	
	Runnable job;
	
	Worker(Runnable job) {
		super(job);
		this.job = job;
	}
	
	public int getResult() {
		return ((Job) job).count();
	}
	
}

class Job implements Runnable {

	private final Predicate<Point> p;
	private final int start;
	private final int end;
	private final List<Point> pool;
	private int count = 0;
	
	Job(List<Point> pool, int start, int end, Predicate<Point> p) {
		this.start = start;
		this.end = end;
		this.p = p;
		this.pool = pool;
	}
	
	public int count() {
		return count;
	}
	
	@Override
	public void run() {
		for (int i = start; i < end; i++) if (p.test(pool.get(i))) count++;
		// System.out.println(Thread.currentThread().getName() + " done.");
	}
	
}